﻿--[[

TODO: Rewrite most of the functions that pull information out of the tooltip.
REQS: 1) Faction Tooltip extractor.
      2) NPC Title extractor.
	 3) One function to create a pet owner line, not five.

]]--

TooltipHandler = {
	skinnableTableBuild = false;
	skinnableTable = {};

	skinnableColours = {};

	guildMembers = {};
	friendEntries = {};
	ignoredList = {};
};

--[[

TooltipHandler:BuildTooltip(status, tooltipBase, tooltipUnit)

-- Builds and shows a tooltip for a known player, unknown player, ignored player, npc, or pet.

]]--

function TooltipHandler:BuildTooltip(status, tooltipBase, tooltipUnit)
	if tooltipBase == "" or tooltipBase == nil then 
		tooltipBase = "GameTooltip";
	end
	if tooltipUnit == "" or tooltipUnit == nil then 
		tooltipUnit = "mouseover"; 
	end
	
	local tt = getglobal(tooltipBase);
	
	for i = 1, tt:NumLines() do
		getglobal(tooltipBase .. "TextLeft" .. i):SetText(nil);
	end
	tt:SetHeight(0);
	
	local tooltipLines = { };
	local original_lines = tt:NumLines();
	
	for i = 1, table.getn(PlayerTooltip[status]) do
		nextLine = self:ParseString(PlayerTooltip[status][i], tooltipUnit);
		
		skinningIsIn = false;
		if nextLine == FlagRSP_Locale_SkinnableLine then
			for i2 = 1, i - 1 do
				FlagRSP:print("." .. nextLine .. "--" .. tooltipLines[i2]);
				if tooltipLines[i2] == nextLine then
					skinningIsIn = true;
				end
			end
		end
		
		if not skinningIsIn then
			tooltipLines[i] = nextLine;
		end
	end
	
	local current_line = 1;
	for t = 1, table.getn(tooltipLines) do
		local ttLeft = getglobal(tooltipBase .. "TextLeft" .. current_line);

		if ( tooltipLines[t] ~= nil and tooltipLines[t] ~= "" ) then
			if ( current_line <= original_lines ) then
				ttLeft:SetText(tooltipLines[t]);
			else
				tt:AddLine(tooltipLines[t], 1, 1, 1);
			end

			current_line = current_line + 1;
		end
	end
	
	tt:Show();
end


--[[

TooltipHandler:checkCondition(type, value, value2, side, linenum, line)

- checks modifier condition <type, value, value2> for line <side, linenum, line>.

]]--
function TooltipHandler:checkCondition(type, value, value2, side, linenum, line, playerID)
	local condition = false;

	if type == "true" then 
		condition = true;
	elseif type == "matchString" then
		if line ~= nil and value ~= "" and line ~= "" then
			value = self:ParseString(value, playerID);
			value = self:removeColourCode(value);
			line = self:removeColourCode(line);
	 
			if value ~= "" then
				if string.find(string.lower(line), string.lower(value)) ~= nil then 
					condition = true; 
				end
			end
		end
	elseif type == "matchExactString" then
		if line ~= nil and value ~= "" and line ~= "" then
			value = self:ParseString(value, playerID);
			value = self:removeColourCode(value);
			line = self:removeColourCode(line);
			
			if value ~= "" then
				if string.lower(line) == string.lower(value) then 
					condition = true; 
				end
			end
		end
	elseif type == "lineNumber" then
		
		if string.lower(value) ~= "last" then
			local vside = string.sub(string.lower(value), 1, 1);
			local vnum = tonumber(string.sub(string.lower(value), 2));
			
			if side == vside and linenum == vnum then
				condition = true;
			end
		end
	end
	
	return condition;
end


--[[

TooltipHandler:modifyLine(type, value, linenum, lines)

- modifies <type, value> for line <linenum> in <lines>.

]]--
function TooltipHandler:modifyLine(type, value, linenum, lines, linesRight, playerID)
	local tline = "";

	if type == "delete" then
		lines[linenum] = "";
		linesRight[linenum] = "";
	elseif type == "replace" then
		tline = self:ParseString(value, playerID);
		if tline ~= "" then 
			lines[linenum] = tline; 
		end
	elseif type == "insertLine" then
		table.insert(lines, linenum, self:ParseString(value, playerID));
		table.insert(linesRight, linenum, " ");
	elseif type == "appendLine" then
		table.insert(lines, linenum+1, self:ParseString(value, playerID));
		table.insert(linesRight, linenum+1, " ");
	elseif type == "colorLine" then
		lines[linenum] = "|cFF" .. value .. lines[linenum] .. "|r" ;
	end

	return lines, linesRight;
end

--[[

TooltipHandler:ParseString(str, playerID)

- Searches the string for tokens and then replaces them.

]]--
function TooltipHandler:ParseString(str, playerID)
	-- Loops through str for tokens and then replaces them with a corresponding function.
	if ( playerID == "" or playerID == nil ) then
		playerID = "mouseover";
	end
	
	for w in string.gmatch(str, "(%%%a+)") do
		local func = TooltipLookup[w];
		local replace = "%" .. w;
		
		-- If a token has no corresponding function, print an error and returns what we've got.
		if ( func == nil ) then
			Print_Chat("Error: Unknown token: " .. w);
			Print_Chat("In string: " .. str);
			return str;
		end
		
		local result = func(playerID);
		
		if ( result == nil ) then
			result = "";
		end
		str = string.gsub(str, replace, result);
	end
	
	return str;
end

--[[

TooltipHandler:ExtractStrings(tooltipBase)

-- Iterates through the lines of a tooltip and extracts them.

]]--

function TooltipHandler:ExtractStrings(tooltipBase)
	if ( tooltipBase == nil or tooltipBase == "" ) then
		tooltipBase = "GameTooltip";
	end
	
	local linesLeft = {};
	local linesRight = {};
	local tlineL;
	local tlineR;
	local i = 1;
	
	for t=1, getglobal(tooltipBase):NumLines() do
		ttextL = getglobal(tooltipBase .. "TextLeft" .. t);
		ttextR = getglobal(tooltipBase .. "TextRight" .. t);
		tlineL = ttextL:GetText();
		tlineR = ttextR:GetText();

		if tlineL == nil or not ttextL:IsVisible() then tlineL = ""; end
		if tlineR == nil or not ttextR:IsVisible() then tlineR = ""; end

		if tlineL ~= "" or tlineR ~= "" then
			linesLeft[i] = tlineL;
			linesRight[i] = tlineR;
			i = i + 1;
		end
	end 
	
	return linesLeft, linesRight;
end


--[[

TooltipHandler:compileString(str)

- Replaces all % values or variables in string str.

]]--
function TooltipHandler:compileString(str, playerID, name)
	local compiledString = str;
	local realm;

	if ( playerID == "" or playerID == nil ) then 
		playerID = "mouseover"; 
	end
   
	if ( name == nil ) then
		name = self:GetCharacterName(playerID, "unit");
	end

	if string.find(compiledString, "%%flagRSPVersion") ~= nil then
		compiledString = string.gsub(compiledString, "%%flagRSPVersion", FlagRSP.VERSION);
	end
	if string.find(compiledString, "%%flagRSPAltLevelEnemy") ~= nil then
		compiledString = string.gsub(compiledString, "%%flagRSPAltLevelEnemy", self:getAlternativeEnemyLevelText(playerID));
	end
	if string.find(compiledString, "%%UnitClass") ~= nil then
		compiledString = string.gsub(compiledString, "%%UnitClass", self:getClass(playerID));
	end
	if string.find(compiledString, "%%UnitRace") ~= nil then
		compiledString = string.gsub(compiledString, "%%UnitRace", self:getRace(playerID));
	end
	if string.find(compiledString, "%%UnitLevel") ~= nil then
		compiledString = string.gsub(compiledString, "%%UnitLevel", self:getLevel(playerID));
	end
	if string.find(compiledString, "%%UnitName") ~= nil then
		compiledString = string.gsub(compiledString, "%%UnitName", name);
	end
	if string.find(compiledString, "%%UnitRank") ~= nil then
		compiledString = string.gsub(compiledString, "%%UnitRank", self:getPVPRank(playerID));
	end
	if string.find(compiledString, "%%UnitTitle") ~= nil then
		compiledString = string.gsub(compiledString, "%%UnitTitle", self:getTitle(name));
	end
	if string.find(compiledString, "%%UnitGuild") ~= nil then
		compiledString = string.gsub(compiledString, "%%UnitGuild", self:getGuild(playerID));
	end
	if string.find(compiledString, "%%flagRSPFriendlistText") ~= nil then
		compiledString = string.gsub(compiledString, "%%flagRSPFriendlistText", self:getFriendlistTooltipText(UnitName(playerID)));
	end
	if string.find(compiledString, "%%flagRSPGuildFriendlistText") ~= nil then
		compiledString = string.gsub(compiledString, "%%flagRSPGuildFriendlistText", self:getFriendlistGuildTooltipText(playerID));
	end
	if string.find(compiledString, "%%flagRSPRPFlag") ~= nil then
		compiledString = string.gsub(compiledString, "%%flagRSPRPFlag", self:getRPTag(UnitName(playerID)));
	end
	if string.find(compiledString, "%%flagRSPPVPRank") ~= nil then
		compiledString = string.gsub(compiledString, "%%flagRSPPVPRank", self:getPVPRank(playerID));
	end
	if string.find(compiledString, "%%flagRSPPetOwnerSubLine") ~= nil then
		compiledString = string.gsub(compiledString, "%%flagRSPPetOwnerSubLine", self:getMouseoverPetOwnerTooltipSubText());
	end
	if string.find(compiledString, "%%flagRSPPetOwnerLine") ~= nil then
		compiledString = string.gsub(compiledString, "%%flagRSPPetOwnerLine", self:getMouseoverPetOwnerTooltipText());
	end
	if string.find(compiledString, "%%flagRSPPetOwner") ~= nil then
		compiledString = string.gsub(compiledString, "%%flagRSPPetOwner", self:getMouseoverPetOwner());
	end
	if string.find(compiledString, "%%flagRSPSurname") ~= nil then
		compiledString = string.gsub(compiledString, "%%flagRSPSurname", self:getSurname(name));
	end
	if string.find(compiledString, "%%flagRSPLevelLine") ~= nil then
		compiledString = string.gsub(compiledString, "%%flagRSPLevelLine", self:getLevelTooltipText(playerID));
	end
	if string.find(compiledString, "%%flagRSPLevel") ~= nil then
		compiledString = string.gsub(compiledString, "%%flagRSPLevel", self:getLevelText(playerID));
	end
	if string.find(compiledString, "%%flagRSPNameLine") ~= nil then
		compiledString = string.gsub(compiledString, "%%flagRSPNameLine", self:getNameTooltipText(playerID));
--		compiledString = string.gsub(compiledString, "%%flagRSPNameLine", self:BuildName(playerID));
	end
	if string.find(compiledString, "%%flagRSPRankLine") ~= nil then
		compiledString = string.gsub(compiledString, "%%flagRSPRankLine", self:getPVPRankTooltipText(playerID));
	end
	if string.find(compiledString, "%%flagRSPRPLine") ~= nil then
		compiledString = string.gsub(compiledString, "%%flagRSPRPLine", self:getRPTooltipText(name, playerID));
	end
	if string.find(compiledString, "%%flagRSPTitleLine") ~= nil then
		compiledString = string.gsub(compiledString, "%%flagRSPTitleLine", self:getTitleTooltipText(playerID));
	end
	if string.find(compiledString, "%%flagRSPGuildLine") ~= nil then
		compiledString = string.gsub(compiledString, "%%flagRSPGuildLine", self:getGuildTooltipText(playerID));
	end
	if string.find(compiledString, "%%flagRSPCivilianLine") ~= nil then
		compiledString = string.gsub(compiledString, "%%flagRSPCivilianLine", self:getCivilianLine(playerID));
	end
	if string.find(compiledString, "%%flagRSPSkinnableLine") ~= nil then
		compiledString = string.gsub(compiledString, "%%flagRSPSkinnableLine", self:getSkinnableLine(playerID));
	end
	if string.find(compiledString, "%%flagRSPCStatus") ~= nil then
		compiledString = string.gsub(compiledString, "%%flagRSPCStatus", FlagHandler:getFlag("CharFlag", name));
	end
	if string.find(compiledString, "%%flagRSPCSText") ~= nil then
		compiledString = string.gsub(compiledString, "%%flagRSPCSText", self:getCharStatusText(name));
	end
	if string.find(compiledString, "%%flagRSPCharStatusLine") ~= nil then
		compiledString = string.gsub(compiledString, "%%flagRSPCharStatusLine", self:getCharStatusTooltipText(name, playerID));
	end
	if string.find(compiledString, "%%FriendlistFriendstateText") ~= nil then
		compiledString = string.gsub(compiledString, "%%FriendlistFriendstateText", Friendlist:GetFriendstateText(name));
	end
	if string.find(compiledString, "%%FriendlistFriendstateColour") ~= nil then
		compiledString = string.gsub(compiledString, "%%FriendlistFriendstateColour", Friendlist:GetFriendstateColour(name));
	end
	if string.find(compiledString, "%%flagRSPPvPFlag") ~= nil then
		compiledString = string.gsub(compiledString, "%%flagRSPPvPFlag", self:TooltipPVPFlagText(playerID));
	end
	if string.find(compiledString, "%%flagRSPResurrectableText") ~= nil then
		compiledString = string.gsub(compiledString, "%%flagRSPResurrectableText", self:TooltipResurrectableText(playerID))
	end
      
	return compiledString;
end

--[[

These next two methods are kinda hackish, no comments for them since they'll be rewritten or removed later.

]]--

function TooltipHandler:TooltipPVPFlagText(playerID)
	if ( UnitIsPVP(playerID) ) then
		return PVP_ENABLED;
	else
		return "";
	end
end

function TooltipHandler:TooltipResurrectableText(playerID)
	if ( UnitIsDead(playerID) and UnitFactionGroup(playerID) == UnitFactionGroup("player") ) then
		return FlagRSP_Locale_ResurrectableText;
	else
		return "";
	end
end

--[[

TooltipHandler:ParseNPCTooltip()

-- Extracts information from an NPC's tooltip that cannot be acquired through functions.

]]--

function TooltipHandler:ParseNPCTooltip()
	local name;
	local title = nil;
	local level;
	local faction = nil;
	local pvpLine;
	local leader = nil;
	local temp;
	local tempWOColor;
	
	-- Clear the local caches.
	self.npcTitle = nil;
	self.npcFaction = nil;
	self.npcLeader = nil;
	self.skinnable = nil;
	
	-- This pulls out some extraneous information, but it'll stay like this until I find something better.
	for i = 1, GameTooltip:NumLines() do
		temp = getglobal("GameTooltipTextLeft"..i):GetText();
		
		if temp ~= nil then
			-- Throw in a quick check to see if something's skinnable.
			if ( string.find(temp, FlagRSP_Locale_SkinnableText) ) then
				self.skinnable = true;
			end
			
			-- Name always comes first.
			if ( i == 1 ) then
				name = temp;
			end
				
			-- Second line can be either NPC title or level.  If it's not level, then it's title.
			if ( i == 2 ) then
				tempWOColor = self:removeColourCode(temp);
				FlagRSP:printDebug(tempWOColor);
				if string.find(temp, FlagRSP_Locale_Level) or 
					tempWOColor == FlagRSP_Locale_Epuny or tempWOColor== FlagRSP_Locale_Puny or tempWOColor == FlagRSP_Locale_Weak or 
					tempWOColor == FlagRSP_Locale_Equal or tempWOColor == FlagRSP_Locale_Strong or tempWOColor == FlagRSP_Locale_Vstrong or 
					tempWOColor == FlagRSP_Locale_Estrong or tempWOColor == FlagRSP_Locale_Impossible or
					string.find(tempWOColor .. " ", FlagRSP_Locale_Epuny) or
					string.find(tempWOColor .. " ", FlagRSP_Locale_Puny) or
					string.find(tempWOColor .. " ", FlagRSP_Locale_Weak) or
					string.find(tempWOColor .. " ", FlagRSP_Locale_Equal) or
					string.find(tempWOColor .. " ", FlagRSP_Locale_Strong) or
					string.find(tempWOColor .. " ", FlagRSP_Locale_Vstrong) or
					string.find(tempWOColor .. " ", FlagRSP_Locale_Estrong) or
					string.find(tempWOColor .. " ", FlagRSP_Locale_Impossible) then
					-- Level line.
					level = temp;
				else
					-- Title line.
					title = temp;
				end
			end
			
			-- If there is a title then line 3 is the level
			if ( i == 3 and title ) then
				level = temp;
			elseif ( i == 3 and temp ~= PVP_ENABLED and not string.find(temp, FlagRSP_Locale_SkinnableText) ) then
				-- If it's the third line and it is not PvP and it is not "Skinnable", then it's a faction.
				faction = temp;
			end

			-- From the fourth line on, the rest could be anything so we simply parse for PVP and skinnable tags.
			-- The rest will be stored in faction. (Even the leader which was stored separatly up to version 1.0.8.)
			if ( i >= 4 and temp == PVP_ENABLED ) then
				pvpLine = temp;
			elseif ( i >= 4 ) and not string.find(temp, FlagRSP_Locale_SkinnableText) then
				if faction == nil then
					faction = temp;
				else
					faction = faction .. "\n" .. temp;
				end
			end
		end
	end
	
	-- Faction may contain a skinnable hint. We will remove this
	
	-- Assign local caches.
	self.npcTitle = title;
	self.npcFaction = faction;
	self.npcLeader = leader;
end

--[[

TooltipHandler:GetNPCTitleLine()

-- Returns the leader line extracted from TooltipHandler:ParseNPCTooltip().

]]--

function TooltipHandler:GetNPCTitleLine()
	return self.npcTitle;
end

--[[

TooltipHandler:GetNPCFactionLine()

-- Returns the faction line extracted from TooltipHandler:ParseNPCTooltip().

]]--

function TooltipHandler:GetNPCFactionLine()
	return self.npcFaction;
end

--[[

TooltipHandler:GetNPCLeaderLine()

-- Returns the leader line extracted from TooltipHandler:ParseNPCTooltip().

]]--

function TooltipHandler:GetNPCLeaderLine()
	return self.npcLeader;
end


--[[

TooltipHandler:getAlternativeEnemyLevelText(playerID)

- Returns alternative level information for enemies.

]]--
function TooltipHandler:getAlternativeEnemyLevelText(playerID)
	local Type, prefix;
	local level;

	if string.sub(playerID,1,5) == "index" then
		local iname = self:GetCharacterName(playerID, "unit");
		level = Friendlist:GetLevel(iname);
	else
		level = UnitLevel(playerID);
	end      

	if level ~= nil and level ~= "" and level ~= 0 then
		Diff = level - UnitLevel("player");

		if string.sub(playerID,1,5) ~= "index" and UnitClassification(playerID) == "elite" then
			Diff = Diff + 5;
		end
      
		local r, g, b;
		if ( level == -1 or Diff >= 7 ) then
			r = QuestDifficultyColors["impossible"].r;
			g = QuestDifficultyColors["impossible"].g;
			b = QuestDifficultyColors["impossible"].b;
		elseif ( Diff >= 4 ) then
			r = QuestDifficultyColors["verydifficult"].r;
			g = QuestDifficultyColors["verydifficult"].g;
			b = QuestDifficultyColors["verydifficult"].b;
		elseif ( Diff >= -1 ) then
			r = QuestDifficultyColors["difficult"].r;
			g = QuestDifficultyColors["difficult"].g;
			b = QuestDifficultyColors["difficult"].b;
		elseif ( -Diff <= GetQuestGreenRange() ) then
			r = QuestDifficultyColors["standard"].r;
			g = QuestDifficultyColors["standard"].g;
			b = QuestDifficultyColors["standard"].b;
		else
			r = QuestDifficultyColors["trivial"].r;
			g = QuestDifficultyColors["trivial"].g;
			b = QuestDifficultyColors["trivial"].b;
		end
      
		prefix = "|cFF" .. FlagRSP:getHexString(r*255,2) .. FlagRSP:getHexString(g*255,2) .. FlagRSP:getHexString(b*255,2);
      
		if ( level == -1 or Diff >= 10 ) then
			Type = prefix .. FlagRSP_Locale_Impossible;
		elseif ( Diff >= 7 ) then
			Type = prefix .. FlagRSP_Locale_Estrong;
		elseif ( Diff >= 4 ) then
			Type = prefix .. FlagRSP_Locale_Vstrong;
		elseif ( Diff >= 2 ) then
			Type = prefix .. FlagRSP_Locale_Strong;
		elseif ( Diff >= -1 ) then
			Type = prefix .. FlagRSP_Locale_Equal;
		elseif ( Diff >= -GetQuestGreenRange()) then
			Type = prefix .. FlagRSP_Locale_Weak;
		elseif ( Diff >= (-4 -GetQuestGreenRange())) then
			Type = prefix .. FlagRSP_Locale_Puny;
		else
			Type = prefix .. FlagRSP_Locale_Epuny;
		end
      
		Type = Type .. "|r";
	else
		Type = "";
	end

	return Type;
end

--[[

TooltipHandler:getAlternateLevelText(playerID)

- Colors the level text the same as in the target frame.

]]--

function TooltipHandler:getAlternateLevelText(playerID)
	local Type, prefix;
	local level;

	if string.sub(playerID,1,5) == "index" then
		local iname = self:GetCharacterName(playerID, "unit");
		level = Friendlist:GetLevel(iname);
	else
		level = UnitLevel(playerID);
	end

	if level ~= nil and level ~= "" and level ~= 0 then

		local r,g,b;
		local color;

		color = GetQuestDifficultyColor(level);
		r = color.r;
		g = color.g;
		b = color.b;

		if ( level == -1 ) then 
			level = "??"
			r=1;
			g=0.13;
			b=0.13;
		end

		prefix = "|cFF" .. FlagRSP:getHexString(r*255,2) .. FlagRSP:getHexString(g*255,2) .. FlagRSP:getHexString(b*255,2);
		Type = prefix .. FlagRSP_Locale_TradLevelLine .. level;

		Type = Type .. "|r";
	else
		Type = "";
	end

	return Type;
end


--[[

TooltipHandler:getFriendlistTooltipText(name)

- Returns text for friend/foe entry of Friendlist in tooltip for player name.

]]--
function TooltipHandler:getFriendlistTooltipText(name)
	local friendstateText = "";
	local r,g,b,text,friendstate;
	friendstate = Friendlist:GetFriendstate(name);

	if friendstate ~= nil then

		text = ""; 
		r,g,b,text = Friendlist:GetFriendstateInfo(friendstate);

		if text ~= nil and text ~= "" then
			friendstateText = "|cFF" .. FlagRSP:getHexString(r*255,2) .. FlagRSP:getHexString(g*255,2) .. FlagRSP:getHexString(b*255,2) .. text .. "|r";     
		end
	end

	return friendstateText;
end


--[[

TooltipHandler:getFriendlistGuildTooltipText(playerID)

- Returns text for friend/foe entry of Friendlist in tooltip for guild.

]]--
function TooltipHandler:getFriendlistGuildTooltipText(playerID)
	local friendstateText = "";
	local r,g,b,text,friendstate;

	local a,b, guildname;
	guildname, a, b = GetGuildInfo(playerID);
   
	friendstate = Friendlist:GetFriendstate(guildname);

	if friendstate ~= nil then
		r,g,b,text = Friendlist:GetFriendstateGuildInfo(friendstate);
      
		friendstateText = "|cFF" .. FlagRSP:getHexString(r*255,2) .. FlagRSP:getHexString(g*255,2) .. FlagRSP:getHexString(b*255,2) .. text .. "|r";     
	end

	return friendstateText;
end


--[[

TooltipHandler:getLevel(playerID)

- Returns level of player.

]]--
function TooltipHandler:getLevel(playerID)
	local Level = "";

	if string.sub(playerID,1,5) == "index" then
		local iname = self:GetCharacterName(playerID, "unit");
		Level = Friendlist:GetLevel(iname);
	else
		if UnitLevel(playerID) >= 1 then
			Level = UnitLevel(playerID);
		else
			Level = "??";
		end
	end

	return Level;
end


--[[

TooltipHandler:isPet:isPet(playerID)

- Returns true if <playerID> is a pet.

]]--
function TooltipHandler:isPet(playerID)
	local isPet = false;

	if (not UnitIsPlayer(playerID)) and (UnitPlayerControlled(playerID)) then isPet = true; end
   
	return isPet;
end


--[[

TooltipHandler:getPetOwner(tooltipLines)

-- Returns the name of the pet's owner.

]]--
function TooltipHandler:getPetOwner(tooltipLines)
	local ownerName = "";
	--local ownerLine = "";

	for t=1, 8 do
		if tooltipLines[t] ~= nil then
			local i = 0;
			while FlagRSP_Locale_MinionLine[i] ~= nil do

				local s, e, match = string.find(tooltipLines[t], FlagRSP_Locale_MinionLine[i]);

				if match ~= nil and match ~= "" then 
					ownerName = match; 
				end
				i = i + 1;
			end
		end
	end
   
	return ownerName;
end


--[[

TooltipHandler:getPetOwnerLine(tooltipLines)

-- Returns the line for the pet's owner.

]]--
function TooltipHandler:getPetOwnerLine(tooltipLines)
	local ownerLine = "";

	for t=1, 8 do
		if tooltipLines[t] ~= nil then
			local i = 0;
			while FlagRSP_Locale_MinionLine[i] ~= nil do
				local s, e, match = string.find(tooltipLines[t], FlagRSP_Locale_MinionLine[i]);
				if match ~= nil and match ~= "" then 
					ownerLine = tooltipLines[t];
				end
				i = i + 1;
			end
		end
	end
   
	return ownerLine;
end

--[[

TooltipHandler:GetPetOwner(str)

-- Extracts the name of the pet's owner from str.

]]--

function TooltipHandler:GetPetOwner(str)
	local s, e, match;
	
	if ( str == nil ) then
		return;
	end
	
	for i = 0, table.getn(FlagRSP_Locale_MinionLine) do
		s, e, match = string.find(str, FlagRSP_Locale_MinionLine[i]);
		
		if ( match ~= nil and match ~= "" ) then
			return match, i;
		end
	end
	
	-- If all else fails, return an empty string.
	return "";
end

--[[

TooltipHandler:BuildPetOwnerLine()

]]--
function TooltipHandler:BuildPetOwnerLine()
	local result, index;
	local ownerLine;
	
	-- By the time this is called, GameTooltip's contents have been destroyed.
	-- So we use a hidden tooltip to scan for the information we need.
	FlagRSPTooltip:SetUnit("mouseover");
	
	for t = 1, GameTooltip:NumLines() do
		local line = getglobal("FlagRSPTooltipTextLeft" .. t)
		local text = line:GetText();
		
		result, index = self:GetPetOwner(text);
		
		if ( result ~= "" ) then
			break;
		end
	end
	
	if ( result == "" ) then
		return "";
	end

	if (FlagRSP_Locale_Minion[index] ~= nil) then
		if ( FlagRSP:getNameDisp() and not Friendlist:isFriend(result) )  then
			-- Pet belongs to an unknown player.  Hide the name.
			ownerLine = string.gsub(FlagRSP_Locale_Minion[index], "%%s", FlagRSP_Locale_Unknown);
		else
			-- Pet owner's name can be shown.
			ownerLine = string.gsub(FlagRSP_Locale_Minion[index], "%%s", result);
		end
	else
		-- In WoW 3.0 this line was called. Usually it shouldn't. Keep an eye on it.
		ownerLine = ""
	end
	
	return ownerLine;
end


--[[

TooltipHandler:getMouseoverPetOwner()

-- Returns the name of the pet's owner.

]]--
function TooltipHandler:getMouseoverPetOwner()
	local Lines = {};
	local ttext;


	for t = 1, 8 do
		ttext = getglobal("GameTooltipTextLeft" .. t);
		local tline = ttext:GetText();
		if tline == nil then 
			tline = ""; 
		end
		Lines[t] = tline;
	end

	return self:getPetOwner(Lines);
end


--[[

TooltipHandler:getMouseoverPetOwnerLine()

-- Returns the owner's tooltip line of the pet.

]]--
function TooltipHandler:getMouseoverPetOwnerLine()
	local Lines = {};
	local ttext;


	for t = 1, 8 do
		ttext = getglobal("GameTooltipTextLeft" .. t);
		local tline = ttext:GetText();
		if tline == nil then 
			tline = ""; 
		end
		Lines[t] = tline;
	end
   
   return self:getPetOwnerLine(Lines);
end


--[[

TooltipHandler:getClass(playerID)

- Returns class of player.

]]--
function TooltipHandler:getClass(playerID)
	local Class = "";
   
	-- capture self-made playerID for Friendlist
	if string.sub(playerID,1,5) == "index" then
		local iname = self:GetCharacterName(playerID, "unit");
		Class = Friendlist:GetClass(iname);
	else
		if UnitClass(playerID) ~= nil then
			-- Lets the class be colored on the tooltip.
			if ( FlagRSPConfigure:GetClassColor() ) then
				local class, file = UnitClass(playerID);
				local color;
				color = RAID_CLASS_COLORS[file];
				
				Class = "|cFF" .. FlagRSP:getHexString(color.r*255,2) .. FlagRSP:getHexString(color.g*255,2) .. FlagRSP:getHexString(color.b*255,2);
				Class = Class .. class .. "|r";
			else
				Class = UnitClass(playerID);
		end
	end
		if UnitCreatureType(playerID) ~= nil and UnitCreatureType(playerID) ~= "" and not UnitIsPlayer(playerID) and not UnitPlayerControlled(playerID) then
			Class = UnitCreatureType(playerID);
		end
		if UnitCreatureFamily(playerID) ~= nil and UnitCreatureFamily(playerID) ~= "" then
			Class = UnitCreatureFamily(playerID);
		end
	end
   
	return Class;
end


--[[

TooltipHandler:getRace(playerID)

- Returns race of player.

]]--
function TooltipHandler:getRace(playerID)
	local Race = "";

	-- capture self-made playerID for Friendlist
	if string.sub(playerID,1,5) == "index" then
		local iname = self:GetCharacterName(playerID, "unit");
		Race = Friendlist:GetRace(iname);
	else
		if UnitRace(playerID) ~= nil then
			Race = UnitRace(playerID);
		end
	end
	
	if ( Race == nil ) then
		Race = "";
	end

	return Race;
end


--[[

TooltipHandler:getName(playerID)

- Returns name of player.

]]--
function TooltipHandler:getName(playerID)
	return self:GetCharacterName(playerID, "first");
end

function TooltipHandler:getFullname(name)
	local fullname = "";
   
	if FlagHandler:getFlag("Fullname", name) ~= "" then
		fullname = FlagHandler:getFlag("Fullname", name);
	elseif Friendlist:getFullname(name) ~= "" and Friendlist:getFullname(name) ~= nil then
		fullname = Friendlist:getFullname(name);
	end
   
	return fullname;
end

--[[

TooltipHandler:getSurname(name)

- Returns surname for player name.

]]--
function TooltipHandler:getSurname(name)
	local surname = "";
   
	if FlagHandler:getFlag("Surname", name) ~= "" then
		surname = FlagHandler:getFlag("Surname", name);
	elseif Friendlist:GetSurname(name) ~= "" and Friendlist:GetSurname(name) ~= nil then
		surname = Friendlist:GetSurname(name);
	end
   
	return surname;
end

--[[

TooltipHandler:GetCharacterName(playerID, position)

-- Returns the type of name for the character and, if possible, the server.
--  These values may change later.
--  Handled types for <position>: unit, first, last, fullname

]]--
function TooltipHandler:GetCharacterName(playerID, position)
	if ( position == nil or position == "" or playerID == nil or playerID == "" ) then
		return;
	end
	
	local name = "";
	
	if (string.sub(playerID, 1, 5) ~= "index" and position == "unit" and UnitName(playerID) ~= nil) then
		local server;
		name, server = UnitName(playerID);
		return name, server;
	elseif ( string.sub(playerID, 1, 5) == "index" and position == "unit" ) then
		name = Friendlist:GetName(playerID);
		return name;
	elseif ( position == "first" ) then
		local temp = self:GetCharacterName(playerID, "unit")
	
		name = FlagHandler:getFlag("Name", temp)
		if ( ( name ~= nil and name ~= " " and name ~= "" ) ) then
			return name;
		elseif ( string.sub(playerID,1,5) == "index" ) then
			name = Friendlist:GetName(playerID, true);
			return name
		else
			if ( UnitName(playerID) ~= nil ) then
				name = UnitName(playerID);
				return name
			end
		end
	elseif ( position == "middle" ) then			
		
	elseif ( position == "last" ) then
		if ( playerID == "mouseover" or playerID == "target" or playerID == "player" ) then
			local temp = self:GetCharacterName(playerID, "unit")

			if ( FlagHandler:getFlag("Surname", temp) ~= "" ) then
				name = FlagHandler:getFlag("Surname", temp);
				return name;
			elseif ( Friendlist:GetSurname(temp) ~= "" and Friendlist:GetSurname(temp) ~= nil ) then
				name = Friendlist:GetSurname(temp);
				return name;
			end
		end
    elseif ( position == "fullname" ) then
        if ( playerID == "mouseover" or playerID == "target" or playerID == "player" ) then
			local temp = self:GetCharacterName(playerID, "unit")

			if ( FlagHandler:getFlag("Fullname", temp) ~= "" ) then
				name = FlagHandler:getFlag("Fullname", temp);
				return name;
			elseif ( Friendlist:GetFullname(temp) ~= "" and Friendlist:GetFullname(temp) ~= nil ) then
				name = Friendlist:GetFullname(temp);
				return name;
			end
        end
	end
end

--[[
TooltipHandler:BuildName(playerID)

- Builds a full character name of the playerID, whether it be from mouseover,
  target, or Friendlist.

]]--
function TooltipHandler:BuildName(playerID)
	local nameText;
	local firstName, unitName, lastName
	
	if ( string.sub(playerID, 1, 5) == "index" ) then
		-- We have a Friendlist entry, build from there.
		unitName, firstName = Friendlist:GetName(playerID);
		
		if ( FlagHandler:getFlag("Surname", unitName) ~= "" ) then
			lastName = FlagHandler:getFlag("Surname", unitName);
		elseif ( Friendlist:GetSurname(unitName) ~= "" and Friendlist:GetSurname(unitName) ~= nil ) then
			lastName = Friendlist:GetSurname(unitName);
		end
		
		-- Okay, we've got our names, let's put them together.
		nameText = ""
		
		if ( firstName ~= "" and firstName ~= nil ) then
			nameText = firstName .. " ";
		end
		if ( unitName ~= nil and unitName~= "" )  then
			nameText = nameText .. unitName;
		end
		
		if ( lastName ~= nil and lastName ~= "" ) then
			if ( string.sub(lastName, 1, 1) == "," ) then
				nameText = nameText .. lastName;
			else
				nameText = nameText .. " " .. lastName;
			end
		end
		
		return nameText;
	elseif ( playerID == "target" or playerID == "mouseover" or playerID == "player" ) then
		-- We have a target or mouseover.
	end
end

--[[

TooltipHandler:getPVPRank(playerID)

- Returns name of player.

]]--
function TooltipHandler:getPVPRank(playerID)
	local Rank = "";

	-- capture self-made playerID for Friendlist
	if string.sub(playerID,1,5) == "index" then
		local iname = self:GetCharacterName(playerID, "unit");
		if ( Friendlist:GetRank(iname) ~= nil ) then
			Rank = Friendlist:GetRank(iname);
		end
	else
		local name = UnitName(playerID);
		if ( UnitPVPName(playerID) ~= name and name ~= nil and UnitPVPName(playerID) ~= nil ) then
			Rank = ( string.sub(UnitPVPName(playerID), 1, string.find(UnitPVPName(playerID), name)-1) );
		end
	end

	return Rank;
end


--[[

TooltipHandler:getGuild(playerID)

- Returns guild of player.

]]--
function TooltipHandler:getGuild(playerID)
	local guild = "";

	-- capture self-made playerID for Friendlist
	if string.sub(playerID,1,5) == "index" then
		local iname = self:GetCharacterName(playerID, "unit");
		guild = Friendlist:GetGuild(iname);
	else
		local a,b;
		guild, a, b = GetGuildInfo(playerID);
	end
	
	if ( guild == nil ) then 
		guild = ""; 
	end

	return guild;
end

--[[

TooltipHandler:getAlternateGuild(playerID)

- Returns guild of player the way I want it to be.

]]--
function TooltipHandler:getAlternateGuild(playerID)
	local guild = "";
   
	-- capture self-made playerID for Friendlist
	if string.sub(playerID,1,5) == "index" then
		local iname = self:GetCharacterName(playerID, "unit");
		guild = Friendlist:GetGuild(iname);
	else
		local temp, rank, index;
		temp, rank, index = GetGuildInfo(playerID);
      
		if ( temp ~= nil and temp ~= "" ) then
			if ( rank ~= nil and rank ~= "" and FlagRSPConfigure:GetGuildRank() ) then
				guild = rank .. " " .. FlagRSP_Locale_Of .. " " .. temp;
			else
				guild = temp;
			end
		else
			guild = "";
		end
	end

	return guild;
end

--[[

TooltipHandler:getTitle(name)

- Returns title for player name.

]]--
function TooltipHandler:getTitle(name)
	local title = "";

	if FlagHandler:getFlag("Title", name) ~= "" then
		title = FlagHandler:getFlag("Title", name);
	elseif Friendlist:GetTitle(name) ~= nil then
		title = Friendlist:GetTitle(name);
	end

	return title;
end


--[[

TooltipHandler:getSkinningSkill()

- Returns skinnung skill for player.

]]--
function TooltipHandler:getSkinningSkill()
	local skill = 0;
	local name, icon, skillLevel, maxSkillLevel, numAbilities, spelloffset, skillLine;
	local profIndex1, profIndex2;
	
	profIndex1, profIndex2 = GetProfessions();
	if profIndex1 ~= nil then
		name, icon, skillLevel, maxSkillLevel, numAbilities, spelloffset, skillLine = GetProfessionInfo(profIndex1);
		if (name ~= FlagRSP_Locale_Skinning) and (profIndex2 ~= nil) then
			name, icon, skillLevel, maxSkillLevel, numAbilities, spelloffset, skillLine = GetProfessionInfo(profIndex2);
		end
			

		if name == FlagRSP_Locale_Skinning then
			skill = skillLevel;
		end
	end

	return skill;
end


--[[

TooltipHandler:getSkinningColor(playerID)

- Returns skinnung skill for player.

]]--
function TooltipHandler:getSkinningColor(playerID)
	local r,g,b;
	local level = UnitLevel(playerID);
	local skillReal = self:getSkinningSkill();

	if ( not FlagRSPConfigure:GetSkinnableFancy() ) then

		local foundColour = "";
      
		local skill = skillReal - math.fmod(skillReal,5);
      
		r = 1; g = 1; b = 1;
      
		local found = false;
      
		local colours = {};
		colours[0] = "grey";
		colours[1] = "green";
		colours[2] = "yellow";
		colours[3] = "orange";
      
		for i=0, 3 do
			if ( level ~= nil and self.skinnableTable[colours[i]][skill] ~= nil ) then
				if not found and self.skinnableTable[colours[i]][skill] >= level then
					foundColour = colours[i];
					found = true;
					break;
				end
			end
		end
      
		if not found or skillReal == 0 then
			r = self.skinnableColours["red"].r;
			g = self.skinnableColours["red"].g;
			b = self.skinnableColours["red"].b;
		else
			r = self.skinnableColours[foundColour].r;
			g = self.skinnableColours[foundColour].g;
			b = self.skinnableColours[foundColour].b;
		end
	else
		local lSkill = skillReal - math.mod(skillReal,5);
		local hSkill = skillReal - math.mod(skillReal,5) + 5;

		local quotSkill = (skillReal - lSkill)*0.2;

		local colours = {};
		colours[1] = "grey";
		colours[2] = "green_a";
		colours[3] = "yellow_a";
		colours[4] = "orange_a";
		colours[5] = "red";

		local colourLevel = {};
      
		for i=1, 4 do
			colourLevel[i] = self.skinnableTable[colours[i]][lSkill] + (self.skinnableTable[colours[i]][hSkill] - self.skinnableTable[colours[i]][lSkill])*quotSkill;
		end      
		-- red is special.
		colourLevel[5] = self.skinnableTable["orange"][lSkill]+1 + (self.skinnableTable["orange"][hSkill] - self.skinnableTable["orange"][lSkill])*quotSkill;
		colourLevel[0] = -1;
		colourLevel[6] = 1000;

		-- find the two colours that are interesting for us.
		local lColour, hColour;
		for i=1, 6 do
			if colourLevel[i] >= level and colourLevel[i-1] < level then
				lColour = i-1;
				hColour = i;
			end
		end

		-- special case: level for lColour is -1. Then  hColour is our colour.
		if colourLevel[lColour] <= -1 then
			r = self.skinnableColours[colours[hColour]].r;
			g = self.skinnableColours[colours[hColour]].r;
			b = self.skinnableColours[colours[hColour]].r;
		elseif colourLevel[hColour] == 1000 then
			r = self.skinnableColours["red"].r;
			g = self.skinnableColours["red"].g;
			b = self.skinnableColours["red"].b;
		else
			local quotColour = 1/(colourLevel[hColour] - colourLevel[lColour]);

			local hQuot, lQuot;

			hQuot = quotColour*(level-colourLevel[lColour]);
			lQuot = quotColour*(colourLevel[hColour]-level);

			r = hQuot*selfskinnableColours[colours[hColour]].r + lQuot*self.skinnableColours[colours[lColour]].r;
			g = hQuot*self.skinnableColours[colours[hColour]].g + lQuot*self.skinnableColours[colours[lColour]].g;
			b = hQuot*self.skinnableColours[colours[hColour]].b + lQuot*self.skinnableColours[colours[lColour]].b;
	 
		end

		if skillReal == 0 then
			r = self.skinnableColours["red"].r;
			g = self.skinnableColours["red"].g;
			b = self.skinnableColours["red"].b;
		end
	end

	if level == -1 then
		r = self.skinnableColours["red"].r;
		g = self.skinnableColours["red"].g;
		b = self.skinnableColours["red"].b;
	end

	return r,g,b;
end


--[[

TooltipHandler:getRPTag(name)

- Returns rp flag for player name (no colours, just the string).

]]--
function TooltipHandler:getRPTag(name)
	local rptag = "";

	if FlagHandler:getFlag("RPFlag", name) ~= "" and FlagHandler:getFlag("RPFlag", name) >= 1 then
		if FlagHandler:getFlag("RPFlag", name) == 1 then
			rptag = FlagRSP_Locale_RP;
		elseif FlagHandler:getFlag("RPFlag", name) == 2 then
			rptag = FlagRSP_Locale_RP2;
		elseif FlagHandler:getFlag("RPFlag", name) == 3 then
			rptag = FlagRSP_Locale_RP3;
		elseif FlagHandler:getFlag("RPFlag", name) == 4 then
			rptag = FlagRSP_Locale_RP4;
		end
	end

	return rptag;
end


-- --[[

-- TooltipHandler:getRP2Tag(name)

-- - Returns rp flag for player name (no colours, just the string).

-- ]]--
-- function TooltipHandler:getRP2Tag(name)
	-- local rptag = "";

	-- if xTP_RP2List[name] ~= nil and xTP_RP2List[name] then
		-- if UList_Installed and UList[FlagRSP.rName] ~= nil and UList[FlagRSP.rName][name] then
			-- rptag = FlagRSP_Locale_UnicornOfficial;
		-- else
			-- rptag = FlagRSP_Locale_UnicornNonOfficial;
		-- end
	-- end
    
	-- return rptag;
-- end


--[[

TooltipHandler:getLevelTooltipText(playerID)

- Returns text for level line in tooltip depending on levelchange.
- Needed: Other level description for friends.

]]--
function TooltipHandler:getLevelTooltipText(playerID)
	local levelText = "";

	local isplayer;
	local iname = self:getName(playerID);

	if string.sub(playerID,1,5) == "index" then
		isplayer = true;
	else
		isplayer = UnitIsPlayer(playerID);
	end

	local level = TooltipHandler:getLevel(playerID);

	if level ~= "" and level ~= 0 then
		if isplayer then
			if ( FlagRSPConfigure:GetLevelDisplay() == "relative" ) then
				levelText = self:getAlternativeEnemyLevelText(playerID) .. " " .. self:getRace(playerID) .. " " .. self:getClass(playerID);
			elseif ( FlagRSPConfigure:GetLevelDisplay() == "color" ) then
				levelText = self:getAlternateLevelText(playerID) .. " " .. self:getRace(playerID) .. " " .. self:getClass(playerID);
			else
				levelText = FlagRSP_Locale_TradLevelLine .. self:getLevel(playerID) .. " " .. self:getRace(playerID) .. " " .. self:getClass(playerID);
			end
		else
			if ( FlagRSPConfigure:GetLevelDisplay() == "relative" ) then
				levelText = FlagRSP_Locale_AltLevelLine .. self:getAlternativeEnemyLevelText(playerID);
			elseif ( FlagRSPConfigure:GetLevelDisplay() == "color" ) then
				levelText = self:getAlternateLevelText(playerID);
			else
				levelText = FlagRSP_Locale_TradLevelLine .. self:getLevel(playerID);
			end

			if ( not UnitIsFriend("player", playerID) and not UnitIsDead(playerID) ) then
				local uct = UnitCreatureType(playerID);
				if uct ~= nil then
					levelText = levelText .. " " .. uct;
				end
			end

			if ( UnitIsDead(playerID) ) then
				levelText = levelText .. FlagRSP_Locale_Corpse;
			end

			if ( string.sub(playerID,1,5) ~= "index" and UnitClassification(playerID) == "elite" ) then
				levelText = levelText .. FlagRSP_Locale_Elite;
			elseif ( string.sub(playerID,1,5) ~= "index" and UnitClassification(playerID) == "worldboss" ) then
				levelText = levelText .. FlagRSP_Locale_Boss;
			elseif ( string.sub(playerID,1,5) ~= "index" and UnitClassification(playerID) == "rare" ) then
				levelText = levelText .. FlagRSP_Locale_Rare;
			elseif ( string.sub(playerID,1,5) ~= "index" and UnitClassification(playerID) == "rareelite" ) then
				levelText = levelText .. FlagRSP_Locale_RareElite;
			end
		end
	end
   
	return levelText;
end


--[[

TooltipHandler:getLevelText(playerID)

- Returns text for level line in tooltip depending on levelchange.
- Needed: Other level description for friends.

]]--
function TooltipHandler:getLevelText(playerID)
	local levelText = "";

	if ( FlagRSPConfigure:GetLevelDisplay() == "relative" ) then 
		levelText = FlagRSP_Locale_AltLevelLine .. self:getAlternativeEnemyLevelText(playerID);
	else
		if self:getLevel(playerID) ~= 0 then
			levelText = FRIENDLIST_LOCALE_LevelLine .. self:getLevel(playerID);
		end
	end
   
	return levelText;
end


--[[

TooltipHandler:updateFriendEntries()

-- Updates the list of standard friend list entries.

]]--
function TooltipHandler:updateFriendEntries()
	self.friendEntries = nil;
	self.friendEntries = {};

	FlagRSP:printDebug("updating friend list.");   
	for t = 1, GetNumFriends() do
		local fName = GetFriendInfo(t);
		self.friendEntries[t] = fName;
	end
end


--[[

TooltipHandler:updateIgnoredList()

-- Updates the list of ignored players.

]]--
function TooltipHandler:updateIgnoredList()
	self.ignoredList = nil;
	self.ignoredList = {};
   
	FlagRSP:printDebug("updating ignored list.");
	for t = 1, GetNumIgnores() do
		local iName = GetIgnoreName(t);

		self.ignoredList[t] = iName;
	end
end


--[[

TooltipHandler:updateGuildMembers()

-- Updates the list of own guild members.

]]--
function TooltipHandler:updateGuildMembers()
	self.guildMembers = nil;
	self.guildMembers = {};
   
	FlagRSP:printDebug("updating guild list.");
	local u = GetNumGuildMembers();
	for t = 1, u do
		local gName = GetGuildRosterInfo(t);

		self.guildMembers[t] = gName;
	end
end


--[[

TooltipHandler:playerIsGuildMember(name)

- Checks, if player <name> is a guild member of the user.
- Returns true if so, false if not.

]]--
function TooltipHandler:playerIsGuildMember(name)
	local isMember = false;

	for i=1, table.getn(self.guildMembers) do
		if self.guildMembers[i] == name then
			isMember = true;
		end
	end

	return isMember;
end


--[[

TooltipHandler:playerIsFriendEntry(name)

- Checks, if player <name> is a guild member of the user.
- Returns true if so, false if not.

]]--
function TooltipHandler:playerIsFriendEntry(name)
	local isFriend = false;

	for i=1, table.getn(self.friendEntries) do
		if self.friendEntries[i] == name then
			isFriend = true;
		end
	end

	return isFriend;
end


--[[

TooltipHandler:playerIsIgnored(name)

- Checks, if player <name> is a guild member of the user.
- Returns true if so, false if not.

]]--
function TooltipHandler:playerIsIgnored(name)
	local isIgnored = false;

	for i=1, table.getn(self.ignoredList) do
		if self.ignoredList[i] == name then
			isIgnored = true;
		end
	end

	return isIgnored;
end


--[[

TooltipHandler:playerIsKnown(name)

- Checks, if player name is known to us (self, friends, ignore, guild, Friendlist).
- Returns true if known, false if unknown.

]]--
function TooltipHandler:playerIsKnown(name)
	local isKnown = false;

	if name == UnitName("player") then 
		isKnown = true; 
	end
   
	if self:playerIsFriendEntry(name) then
		isKnown = true;
	end

	if 1 == 0 then
		for t = 1, GetNumFriends() do
			local fName = GetFriendInfo(t);
			if fName == name then 
				isKnown = true;
				break; 
			end
		end
	end

	if self:playerIsIgnored(name) then
		isKnown = true;
	end

	if 1 == 0 then
		for t = 1, GetNumIgnores() do
			local iName = GetIgnoreName(t);
			if iName == name then 
				isKnown = true; 	
				break; 
			end
		end
	end

	if self:playerIsGuildMember(name) then
		isKnown = true;
	end

	if 1 == 0 then
		local u = GetNumGuildMembers();

		for t = 1, u do
			local gName = GetGuildRosterInfo(t);
			if gName == name then 
				isKnown = true;
				break; 
			end
		end
	end
   
	if Friendlist:isFriend(name) then 
		isKnown = true;
	end

	return isKnown;
end


--[[

TooltipHandler:getNameTooltipText(playerID)

- Returns text for name line in tooltip.

]]--
function TooltipHandler:getNameTooltipText(playerID)
	local nameText = "";
	local prefix;
   
	local r, g, b = self:getTooltipNameColor(playerID);
	local colorprefix = "|cFF" .. FlagRSP:getHexString(r*255,2) .. FlagRSP:getHexString(g*255,2) .. FlagRSP:getHexString(b*255,2);
	prefix = colorprefix;

	--local name, server = UnitName(playerID);
	local name, server = self:GetCharacterName(playerID, "unit");
    
    local displayedName = "";
	local firstName = self:GetCharacterName(playerID, "first");
   
    local fullname = self:GetCharacterName( playerID, "fullname");
    if ( fullname ~= nil and fullname ~= "" and fullname ~= " " ) then-- and name ~= fullname ) then
        prefix = prefix .. fullname .. " ";
    elseif ( fullname == nil and server ~= nil and server ~= "" ) then
        prefix = prefix .. name .. " " .. FlagRSP_Locale_Of .. " " .. server .. " (*)";
    end
	
	if name == nil then
		-- To prevent an error I have not solved yet.
		name = "";
	end    
    
    -- Real player
	if UnitIsPlayer(playerID) then
		local lastName = self:GetCharacterName(playerID, "last");

		if ( (fullname == nil or fullname == "") and lastName ~= nil and lastName ~= "" and lastName ~= " " and name ~= lastName )  then
			if ( string.sub(lastName, 1, 1) == "," ) then
				nameText = prefix .. name .. lastName;
			else
				nameText = prefix .. name .. " " .. lastName;
			end
        elseif ( fullname == nil ) then
            if ( server ~= nil and server ~= "" ) then -- currently always set for foreign servers i guess
                nameText = prefix .. name .. " " .. FlagRSP_Locale_Of .. " " .. server .. " (*)";
            else
                nameText = prefix .. UnitName(playerID);       
            end     
		elseif ( fullname == name ) then
            nameText = prefix;
        else
            -- if ( server ~= nil and server ~= FlagRSP.rName ) then
                -- name = name .. "-" .. server;
            -- end
            nameText = prefix .. " \n|cff909090(" .. name .. ")"; -- player real name
		end
		
		if ( server ~= nil and server ~= "" ) then
            nameText = prefix .. " \n|cff909090(" .. name .. " " .. FlagRSP_Locale_Of .. " " .. server .. ")";
		end

		nameText = nameText .. "|r";
	elseif self:isPet(playerID) then
		nameText = prefix .. name .. "|r";
	else
		nameText = prefix .. name .. "|r";
	end

	if FlagRSP:getNameDisp() then
		-- Reset prefix
		prefix = colorprefix;
		if not self:playerIsKnown(name) then
			if UnitIsPlayer(playerID) then 
				nameText = prefix .. FlagRSP_Locale_Unknown .. "|r";
                -- Add real player name to the tooltip
                nameText = nameText .. "\n|cff909090(" .. name .. ")"; -- player real name
			elseif self:isPet(playerID) then
				if playerID == "mouseover" then
					if self:playerIsKnown(self:getMouseoverPetOwner()) then
						if self:getMouseoverPetOwner() ~= FlagRSP.pName then	       
							nameText = prefix .. "<" .. self:getMouseoverPetOwnerTooltipText() .. ">|r";
						end
					else
						nameText = prefix .. FlagRSP_Locale_UnknownPet .. "|r";
					end
				else
					nameText = prefix .. FlagRSP_Locale_UnknownPet .. "|r";
				end
			end
		end	    
	end
	
	return nameText;
end


--[[

TooltipHandler:getMouseoverPetOwnerTooltipText()

- Returns text for pet owner line in tooltip.

]]--
function TooltipHandler:getMouseoverPetOwnerTooltipText()
	local ownerline = "";
	local ownerName, origLine;

	if self:getMouseoverPetOwner() ~= "" then
		ownerline = self:getMouseoverPetOwnerLine();
	end
   
	return ownerline;
end


--[[

TooltipHandler:getMouseoverPetOwnerTooltipText()

- Returns text for pet owner subline in tooltip.

]]--
function TooltipHandler:getMouseoverPetOwnerTooltipSubText()
	local line = "";

	if not FlagRSP:getNameDisp() or self:playerIsKnown(UnitName("mouseover")) or self:getMouseoverPetOwner() == UnitName("player") then
		line = self:getMouseoverPetOwnerTooltipText();
	end

	return line;
end


--[[

TooltipHandler:getPVPRankTooltipText(playerID)

- Returns text for pvp rank text in tooltip.

]]--
function TooltipHandler:getPVPRankTooltipText(playerID)
	local pvpRankText = "";

	-- capture self-made playerID for Friendlist
	if string.sub(playerID,1,5) == "index" then
		if self:getPVPRank(playerID) ~= "" then
			pvpRankText = FlagRSP_Locale_PVPRankLine .. self:getPVPRank(playerID);
		end
	else
		if UnitIsPlayer(playerID) and FlagRSPConfigure:GetPVPRanks() and self:getPVPRank(playerID) ~= "" then
			pvpRankText = FlagRSP_Locale_PVPRankLine .. self:getPVPRank(playerID);
		end
	end

	return pvpRankText;
end


--[[

TooltipHandler:getTitleTooltipText()

- Returns text for title line in tooltip.

]]--
function TooltipHandler:getTitleTooltipText(playerID)
	local titleText = "";

	local name = UnitName(playerID);

	if UnitIsPlayer(playerID) and self:getTitle(name) ~= "" then
		titleText = FLAGRSP_TITLECOLORSTRING .. self:getTitle(name) .. "|r";
	end
   
	return titleText;
end


--[[

TooltipHandler:getRPTooltipText(name)

- Returns text for rp tag line in tooltip for player name.

]]--
function TooltipHandler:getRPTooltipText(name, playerID)
	local RPTag = "";

	if playerID == nil then
		playerID = "";
	end

	if ((playerID == "target" or playerID == "mouseover") and (UnitIsPlayer(playerID) or UnitPlayerControlled(playerID))) or (playerID ~= "target" and playerID ~= "mouseover") then
      
		if FlagHandler:getFlag("RPFlag", name) ~= "" and FlagHandler:getFlag("RPFlag", name) >= 1 then
			RPTag = "|cFF" .. FlagRSP:getHexString(1*255,2) .. FlagRSP:getHexString(1*255,2) .. FlagRSP:getHexString(0.5*255,2);

			RPTag = RPTag .. self:getRPTag(name);

			RPTag = RPTag .. "|r";
		end
	end

	return RPTag;
end


-- --[[

-- TooltipHandler:getRP2TooltipText(name)

-- - Returns text for rp tag line in tooltip for player name.

-- ]]--
-- function TooltipHandler:getRP2TooltipText(name, playerID)
	-- local RPTag = "";

	-- if playerID == nil then
		-- playerID = "";
	-- end

	-- if ((playerID == "target" or playerID == "mouseover") and (UnitIsPlayer(playerID) or UnitPlayerControlled(playerID))) or (playerID ~= "target" and playerID ~= "mouseover") then
      
		-- if xTP_RP2List[name] ~= nil and xTP_RP2List[name] then
			-- if UList_Installed and UList[FlagRSP.rName] ~= nil and UList[FlagRSP.rName][name] then
				-- RPTag = "|cFF" .. FlagRSP:getHexString(1*255,2) .. FlagRSP:getHexString(0.667*255,2) .. FlagRSP:getHexString(0.1*255,2);
			-- else
				-- RPTag = "|cFF" .. FlagRSP:getHexString(0.33*255,2) .. FlagRSP:getHexString(0.33*255,2) .. FlagRSP:getHexString(0.33*255,2);
			-- end
	 
			-- RPTag = RPTag .. self:getRP2Tag(name);

			-- RPTag = RPTag .. "|r";
		-- end
	-- end
   
	-- return RPTag;
-- end


--[[

TooltipHandler:getCharStatusText(name)

- Returns text for character status for player <name>.

]]--
function TooltipHandler:getCharStatusText(name)
	local charstatus = "";

    -- local cstatus = self:getCharStatus(name);
    local cstatus = FlagHandler:getFlag("CharFlag", name);
    cstatus = FlagRSP:transformCharFlag(cstatus);
	if cstatus ~= "" then
		charstatus = "|cFF" .. FlagRSP:getHexString(1*255,2) .. FlagRSP:getHexString(1*255,2) .. FlagRSP:getHexString(0.5*255,2);
    
		if cstatus == "ooc" then
			charstatus = charstatus .. FlagRSP_Locale_OOC;
		elseif cstatus == "ic" then
			charstatus = charstatus .. FlagRSP_Locale_IC;
		elseif cstatus == "ffa-ic" then
			charstatus = charstatus .. FlagRSP_Locale_FFAIC;
		elseif cstatus == "st" then
			charstatus = charstatus .. FlagRSP_Locale_ST;
		end

		charstatus = charstatus .. "|r";
	end
   
	return charstatus;
end


--[[

TooltipHandler:getCivilianLine(playerID)

- Returns colored civilian line.

]]--
function TooltipHandler:getCivilianLine(playerID)
	local prefix, line;

	prefix = "|cFF" .. FlagRSP:getHexString(0*255,2) .. FlagRSP:getHexString(1*255,2) .. FlagRSP:getHexString(0*255,2);

	line = prefix .. FlagRSP_Locale_CivilianLine .. "|r";

	return line;
end


--[[

TooltipHandler:getSkinnableLine(playerID)

- Returns colored skinnable line.

]]--
function TooltipHandler:getSkinnableLine(playerID)
	local prefix, line;
	local r,g,b;

	FlagRSP:printDebug(self:getSkinningSkill());
	if ( not self.skinnable ) then
		return;
	end

	r, g, b = self:getSkinningColor(playerID);
   
	prefix = "|cFF" .. FlagRSP:getHexString(r*255,2) .. FlagRSP:getHexString(g*255,2) .. FlagRSP:getHexString(b*255,2);

	line = prefix .. FlagRSP_Locale_SkinnableLine .. "|r";

	return line;
end


--[[

TooltipHandler:getCharStatusTooltipText(name, playerID)

- Returns text for character status for tooltip line for player <name>.

]]--
function TooltipHandler:getCharStatusTooltipText(name, playerID)
	local charstatus = "";

	if ((playerID == "target" or playerID == "mouseover") and (UnitIsPlayer(playerID) or UnitPlayerControlled(playerID))) or (playerID ~= "target" and playerID ~= "mouseover") then
		local cs = self:getCharStatusText(name);
		if cs ~= "" then
			charstatus = FlagRSP_Locale_CharStatusLine .. cs;
		end
	end

	return charstatus;
end



--[[

TooltipHandler:getGuildTooltipText(playerID)

- Returns text for guild line in tooltip.

]]--
function TooltipHandler:getGuildTooltipText(playerID)
	local guildText = "";
	local guild = self:getGuild(playerID);


	if UnitIsPlayer(playerID) and FlagRSP:getGuildDisp() == 1 and guild ~= "" then
		guildText = self:getAlternateGuild(playerID);
	elseif UnitIsPlayer(playerID) and FlagRSP:getGuildDisp() == 0 and guild ~= "" then
		if Friendlist:isFriend(guild) and Friendlist:GetType(guild) == "guild" then 
			guildText = self:getAlternateGuild(playerID);
		end
	end
   
	return guildText;
end


--[[

TooltipHandler:getTooltipBGColor(playerID)

- Returns background color for tooltip.

]]--
function TooltipHandler:getTooltipBGColor(playerID)
	local r, g, b;
	r = 0.09;
	g = 0.09;
	b = 0.19;

	local reaction = UnitReaction(playerID, "player");

	if reaction ~= nil then   
		if not UnitIsPlayer(playerID) and not UnitPlayerControlled(playerID) then
			if reaction >= 5 then r = 0.09; g = 0.19; b = 0.09; end
			if reaction < 4 then r = 0.10; g = 0.09; b = 0.09; end
			if reaction == 4 then r = 0.19; g = 0.19; b = 0.09; end
		elseif UnitIsPVP(playerID) then
			if reaction >= 5 then r = 0.09; g = 0.19; b = 0.1; end
			if reaction < 4 and not UnitIsPVP("player") then
				r = 0.19; g = 0.19; b = 0.09;
			elseif reaction < 4 and UnitIsPVP("player") then
				r = 0.19; g = 0.09; b = 0.09;
			end
			if reaction == 4 then r = 0.19; g = 0.19; b = 0.09; end
		else
			r = 0.09; g = 0.09; b = 0.19;
		end
	else
		if not UnitIsPlayer(playerID) and not UnitPlayerControlled(playerID) then
			r = 0.19; g = 0.19; b = 0.09;
		else
			r = 0.09; g = 0.09; b = 0.19;
		end
	end

	return r, g, b;
end


--[[

TooltipHandler:getTooltipNameColor(playerID)

- Returns text color for tooltip name line.

]]--
function TooltipHandler:getTooltipNameColor(playerID)
	local r,g,b;

	r = 1;
	g = 1;   
	b = 1;

	local reaction = UnitReaction(playerID, "player");
   
	if reaction ~= nil then
		if not UnitIsPlayer(playerID) and not UnitPlayerControlled(playerID) then
			if reaction >= 5 then r = 0; g = 0.95; b = 0; end
			if reaction < 4 then r = 0.95; g = 0; b = 0; end
			if reaction == 4 then r = 0.95; g = 0.95; b = 0; end
		elseif UnitIsPVP(playerID) then
			if reaction >= 5 then r = 0; g = 0.95; b = 0; end
			if reaction < 4 and not UnitIsPVP("player") then
				r = 0.95; g = 0.95; b = 0.1; 
			elseif reaction < 4 and UnitIsPVP("player") then
				r = 0.95; g = 0; b = 0; 
			end
			if reaction == 4 then r = 0.95; g = 0.95; b = 0; end
		else
			r = 0.5; g = 0.5; b = 0.95;
		end
	else
		if not UnitIsPlayer(playerID) and not UnitPlayerControlled(playerID) then
			r = 0.95; g = 0.95; b = 0;
		else
			r = 0.5; g = 0.5; b = 0.95;
		end
	end
   
	return r,g,b;
end


--[[

TooltipHandler:buildSkinnableTable()

- Returns text color for tooltip name line.

]]--
function TooltipHandler:buildSkinnableTable()
	-- build standard table.
	local treshold = {};
	treshold["grey"] = 95;
	treshold["green"] = 45;
	treshold["yellow"] = 20;
	treshold["orange"] = -5;
	local i, d1, d2;

	for colour, t in pairs(treshold) do
		d1 = 19-t/5;
		d2 = t/5+1;
		self.skinnableTable[colour] = {};

		for i=0, 75 do
			if ( t >= (i * 5) ) then
				self.skinnableTable[colour][i*5] = -1;
			elseif ( t < (i * 5) ) then
				if ( (i + d1) <= 40 ) then
					if math.fmod(((i+d1)*5), 10) == 0 then	    
						self.skinnableTable[colour][i*5] = (i+d1)/2;
					else
						self.skinnableTable[colour][i*5] = (i+d1-1)/2;
					end
				else
					self.skinnableTable[colour][i*5] = i-d2;	  
				end
			end
		end
	end
   


	-- build dynamic table.
	self.skinnableTable["green_a"] = {};
	self.skinnableTable["yellow_a"] = {};
	self.skinnableTable["orange_a"] = {};
	for i=0, 65 do 
		self.skinnableTable["green_a"][i*5] = 0.5*(self.skinnableTable["green"][i*5] + self.skinnableTable["grey"][i*5] + 1);
		self.skinnableTable["yellow_a"][i*5] = 0.5*(self.skinnableTable["yellow"][i*5] + self.skinnableTable["green"][i*5] + 1);
		self.skinnableTable["orange_a"][i*5] = 0.5*(self.skinnableTable["orange"][i*5] + self.skinnableTable["yellow"][i*5] + 1);
	end

	-- colours.
	self.skinnableColours["grey"]= {};
	self.skinnableColours["green"]= {};
	self.skinnableColours["yellow"]= {};
	self.skinnableColours["orange"]= {};
	self.skinnableColours["green_a"]= {};
	self.skinnableColours["yellow_a"]= {};
	self.skinnableColours["orange_a"]= {};
	self.skinnableColours["red"]= {};

	self.skinnableColours["grey"].r = 0.5;
	self.skinnableColours["grey"].g = 0.5;
	self.skinnableColours["grey"].b = 0.5;

	self.skinnableColours["green"].r = 0.25;
	self.skinnableColours["green"].g = 0.75;
	self.skinnableColours["green"].b = 0.25;

	self.skinnableColours["yellow"].r = 1;
	self.skinnableColours["yellow"].g = 1;
	self.skinnableColours["yellow"].b = 0;

	self.skinnableColours["orange"].r = 1;
	self.skinnableColours["orange"].g = 0.5;
	self.skinnableColours["orange"].b = 0.25;

	self.skinnableColours["green_a"].r = 0.25;
	self.skinnableColours["green_a"].g = 0.75;
	self.skinnableColours["green_a"].b = 0.25;

	self.skinnableColours["yellow_a"].r = 1;
	self.skinnableColours["yellow_a"].g = 1;
	self.skinnableColours["yellow_a"].b = 0;

	self.skinnableColours["orange_a"].r = 1;
	self.skinnableColours["orange_a"].g = 0.5;
	self.skinnableColours["orange_a"].b = 0.25;

	self.skinnableColours["red"].r = 1;
	self.skinnableColours["red"].g = 0;
	self.skinnableColours["red"].b = 0;
end


--[[

TooltipHandler:removeColourCode(s)

-- removes colour codes from string <s>.

]]--
function TooltipHandler:removeColourCode(s)
	--[0-9,a-f][0-9,a-f][0-9,a-f][0-9,a-f][0-9,a-f][0-9,a-f][0-9,a-f][0-9,a-f]
	local sNew = s;

	if sNew ~= nil then
		while string.find(sNew, "|[c,C][0-9,a-f,A-F][0-9,a-f,A-F][0-9,a-f,A-F][0-9,a-f,A-F][0-9,a-f,A-F][0-9,a-f,A-F][0-9,a-f,A-F][0-9,a-f,A-F]") ~= nil do
			sNew = string.gsub(sNew,"|[c,C][0-9,a-f,A-F][0-9,a-f,A-F][0-9,a-f,A-F][0-9,a-f,A-F][0-9,a-f,A-F][0-9,a-f,A-F][0-9,a-f,A-F][0-9,a-f,A-F]", "");
		end
		while string.find(sNew, "|[r,R]") ~= nil do
			sNew = string.gsub(sNew,"|[r,R]", "");
		end
	end
   
	return sNew;
end